/*
 * Sentinella
 * http://sourceforge.net/projects/sentinella/
 * Copyright (c) 2009 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "MainWindow.h"
#include "MainWindow.moc"

MainWindow::MainWindow(const KAboutData& aboutData, KCmdLineArgs* cmdLineArgs, const short cmdSelectedCondition, const short cmdSelectedAction) :
	KMainWindow(), aboutData(aboutData), cmdLineArgs(cmdLineArgs) {
	condition = NULL;
	action = NULL;
	pidProcessDies = pidKillProcess = 0;
	availableSounds = NULL;
	supportedShutdown = KWorkSpace::canShutDown(KWorkSpace::ShutdownConfirmNo,
			KWorkSpace::ShutdownTypeHalt, KWorkSpace::ShutdownModeForceNow);
	supportedSleepStates = Solid::PowerManagement::supportedSleepStates();
	config = KGlobal::config();

	gui.setupUi(this);
	trayIcon.setAssociatedWidget(this);
	trayIcon.setCategory(KStatusNotifierItem::ApplicationStatus);
	trayIcon.setStatus(KStatusNotifierItem::Active);
	trayIcon.contextMenu()->addAction(KStandardAction::aboutApp(this, SLOT(showAboutApp()),
			trayIcon.contextMenu()));
	trayIcon.setIconByName("Sentinella");

	connect(gui.radioButtonCPU, SIGNAL(toggled(bool)), this, SLOT(conditionRadioButtonToggled(bool)));
	connect(gui.radioButtonMemory, SIGNAL(toggled(bool)), this, SLOT(conditionRadioButtonToggled(bool)));
	connect(gui.radioButtonNetwork, SIGNAL(toggled(bool)), this, SLOT(conditionRadioButtonToggled(bool)));
	connect(gui.radioButtonDateTime, SIGNAL(toggled(bool)), this, SLOT(conditionRadioButtonToggled(bool)));
	connect(gui.radioButtonProcessDies, SIGNAL(toggled(bool)), this, SLOT(conditionRadioButtonToggled(bool)));
	connect(gui.pushButtonProcessDies, SIGNAL(clicked(bool)), this, SLOT(programDiesClicked()));

	actionsGroup.addButton(gui.radioButtonShutdown); // I need this group to fix a GUI bug (the radio buttons can be unchecked clicking on them)
	actionsGroup.addButton(gui.radioButtonReboot);
	actionsGroup.addButton(gui.radioButtonSleep);
	actionsGroup.addButton(gui.radioButtonExecuteCommand);
	actionsGroup.addButton(gui.radioButtonPlayAlarm);
	actionsGroup.addButton(gui.radioButtonKillProcess);
	if (supportedShutdown) {
		connect	(gui.radioButtonShutdown, SIGNAL(toggled(bool)), this, SLOT(actionRadioButtonToggled(bool)));
		connect(gui.radioButtonReboot, SIGNAL(toggled(bool)), this, SLOT(actionRadioButtonToggled(bool)));
	} else {
		gui.radioButtonShutdown->setEnabled(false);
		gui.radioButtonReboot->setEnabled(false);
	}
	if (supportedSleepStates.empty())
		gui.radioButtonSleep->setEnabled(false);
	else
		connect(gui.radioButtonSleep, SIGNAL(toggled(bool)), this, SLOT(actionRadioButtonToggled(bool)));
	connect(gui.radioButtonExecuteCommand, SIGNAL(toggled(bool)), this, SLOT(actionRadioButtonToggled(bool)));
	connect(gui.radioButtonPlayAlarm, SIGNAL(toggled(bool)), this, SLOT(actionRadioButtonToggled(bool)));
	connect(gui.radioButtonKillProcess, SIGNAL(toggled(bool)), this, SLOT(actionRadioButtonToggled(bool)));
	connect(gui.pushButtonKillProcess, SIGNAL(clicked(bool)), this, SLOT(killProcessClicked()));

	connect(gui.pushButtonStartStop, SIGNAL(clicked(bool)), this, SLOT(startStopClicked()));

	initConditionWidgets(cmdSelectedCondition);
	initActionWidgets(cmdSelectedAction);
	if (cmdLineArgs->isSet("start"))
		gui.pushButtonStartStop->click();

	cmdLineArgs->clear(); // the given cmd args are not needed anymore
}

MainWindow::~MainWindow() {
	delete availableSounds;
}

void MainWindow::initConditionWidgets(const short cmdSelectedCondition) {
	int aux;
	if (cmdSelectedCondition)
		aux = cmdSelectedCondition;
	else {
		KConfigGroup group = config->group("Conditions");
		aux = group.readEntry("LastExecuted", 1);
	}

	switch (aux) {
	case 1:
		gui.radioButtonCPU->setChecked(true);
		break;
	case 2:
		gui.radioButtonMemory->setChecked(true);
		break;
	case 3:
		gui.radioButtonNetwork->setChecked(true);
		break;
	case 4:
		gui.radioButtonDateTime->setChecked(true);
		break;
	default:
		gui.radioButtonProcessDies->setChecked(true);
	}

	if (cmdLineArgs->isSet("higher"))
		aux = 0;
	else if (cmdLineArgs->isSet("lower"))
		aux = 1;
	else
		aux = -1;

	KConfigGroup groupCPU = config->group("ConditionCPU");
	gui.comboBoxCPU->setCurrentIndex(aux != -1 ? aux : groupCPU.readEntry("Higher", 1));
	gui.spinBoxCPU->setValue(cmdSelectedCondition == 1 ? cmdLineArgs->getOption("cpu").toInt()
			: groupCPU.readEntry("Percentage", 10));
	gui.timeEditCPU->setTime(QTime::fromString(cmdSelectedCondition == 1 && cmdLineArgs->isSet(
			"during") ? cmdLineArgs->getOption("during") : groupCPU.readEntry("Time", "00:01:00")));

	KConfigGroup groupMemory = config->group("ConditionMemory");
	gui.comboBoxMemory->setCurrentIndex(aux != -1 ? aux : groupMemory.readEntry("Higher", 1));
	gui.spinBoxMemory->setValue(
			cmdSelectedCondition == 2 ? cmdLineArgs->getOption("memory").toInt()
					: groupMemory.readEntry("Amount", 512));
	gui.timeEditMemory->setTime(QTime::fromString(cmdSelectedCondition == 2 && cmdLineArgs->isSet(
			"during") ? cmdLineArgs->getOption("during")
			: groupMemory.readEntry("Time", "00:01:00")));

	KConfigGroup groupNetwork = config->group("ConditionNetwork");
	gui.comboBoxNetworkDirection->setCurrentIndex(groupNetwork.readEntry("Direction", 0));
	gui.comboBoxNetworkHigher->setCurrentIndex(aux != -1 ? aux
			: groupNetwork.readEntry("Higher", 0));
	gui.spinBoxNetwork->setValue(
			cmdSelectedCondition == 3 ? cmdLineArgs->getOption("network").toInt()
					: groupNetwork.readEntry("Amount", 10));
	gui.timeEditNetwork->setTime(QTime::fromString(cmdSelectedCondition == 3 && cmdLineArgs->isSet(
			"during") ? cmdLineArgs->getOption("during") : groupNetwork.readEntry("Time",
			"00:01:00")));

	QDateTime dateTime;
	if (cmdSelectedCondition == 4)
		dateTime = QDateTime::fromString(cmdLineArgs->getOption("date-time"), Qt::ISODate);
	if (cmdSelectedCondition != 4 || !dateTime.isValid()) {
		KConfigGroup groupDateTime = config->group("ConditionDateTime");
		dateTime = groupDateTime.readEntry("DateTime", QDateTime::currentDateTime());
	}
	QTime readTime(dateTime.time().hour(), dateTime.time().minute(), 0); // Setting 0 seconds
	dateTime.setTime(readTime);
	gui.kdatetimewidgetDateTime->setDateTime(dateTime);

	if (cmdSelectedCondition == 5) {
#ifdef SA_OPEN_PROCESS
		if (sa_open_process() != 0)
			return;
#endif
		struct sa_process procInfo;
		if (sa_get_process(cmdLineArgs->getOption("program-dies").toInt(), &procInfo) == 0) {
			pidProcessDies = procInfo.pid;
			gui.labelProcessDies->setText(QString::fromAscii(procInfo.filename));
		}
#ifdef SA_CLOSE_PROCESS
		sa_close_cpu();
#endif
	}
}

void MainWindow::initActionWidgets(const short cmdSelectedAction) {
	int chosen;
	if (cmdSelectedAction)
		chosen = cmdSelectedAction;
	else {
		KConfigGroup group = config->group("Actions");
		chosen = group.readEntry("LastExecuted", 1);
	}

	if (chosen == 1 && supportedShutdown)
		gui.radioButtonShutdown->setChecked(true);
	else if (chosen == 2 && supportedShutdown)
		gui.radioButtonReboot->setChecked(true);
	else if (chosen == 3 && !supportedSleepStates.empty())
		gui.radioButtonSleep->setChecked(true);
	else if (chosen == 4)
		gui.radioButtonExecuteCommand->setChecked(true);
	else if (chosen == 5)
		gui.radioButtonPlayAlarm->setChecked(true);
	else
		gui.radioButtonKillProcess->setChecked(true);

	if (cmdSelectedAction == 1 || cmdSelectedAction == 2)
		gui.checkBoxShutdownForce->setChecked(cmdLineArgs->getOption(
				cmdSelectedAction == 1 ? "shutdown" : "reboot") == "yes");
	else {
		KConfigGroup groupShutdown = config->group("ActionShutdown");
		gui.checkBoxShutdownForce->setChecked(groupShutdown.readEntry("Force", true));
	}

	if (!supportedSleepStates.empty()) {
		gui.radioButtonSleepStandBy->setEnabled(supportedSleepStates.contains(
				Solid::PowerManagement::StandbyState));
		gui.radioButtonSleepSuspend->setEnabled(supportedSleepStates.contains(
				Solid::PowerManagement::SuspendState));
		gui.radioButtonSleepHibernate->setEnabled(supportedSleepStates.contains(
				Solid::PowerManagement::HibernateState));

		Solid::PowerManagement::SleepState state;
		if (cmdSelectedAction == 3) { // cmd arg
			QString cmdSleepArg = cmdLineArgs->getOption("sleep");
			if (cmdSleepArg == "Standby")
				state = Solid::PowerManagement::StandbyState;
			else if (cmdSleepArg == "Suspend")
				state = Solid::PowerManagement::SuspendState;
			else
				state = Solid::PowerManagement::HibernateState;
		} else { // config
			KConfigGroup groupSleep = config->group("ActionSleep");
			state = (Solid::PowerManagement::SleepState) groupSleep.readEntry("Mode",
					(int) Solid::PowerManagement::HibernateState);
		}

		if (!supportedSleepStates.contains(state)) { // Just in case the mode read from the config or cmd arg is unsupported
			QSetIterator<Solid::PowerManagement::SleepState> i(supportedSleepStates);
			state = i.next();
		}

		switch (state) {
		case Solid::PowerManagement::StandbyState:
			gui.radioButtonSleepStandBy->setChecked(true);
			break;
		case Solid::PowerManagement::SuspendState:
			gui.radioButtonSleepSuspend->setChecked(true);
			break;
		default:
			gui.radioButtonSleepHibernate->setChecked(true);
		}
	}

	if (cmdSelectedAction == 6) {
#ifdef SA_OPEN_PROCESS
		if (sa_open_process() != 0)
			return;
#endif
		struct sa_process procInfo;
		if (sa_get_process(cmdLineArgs->getOption("terminate-program").toInt(), &procInfo) == 0) {
			pidKillProcess = procInfo.pid;
			gui.labelKillProcess->setText(QString::fromAscii(procInfo.filename));
		}
#ifdef SA_CLOSE_PROCESS
		sa_close_cpu();
#endif
	}

	KConfigGroup groupExecuteCommand = config->group("ActionExecuteCommand");
	gui.lineEditExecuteCommand->setText(groupExecuteCommand.readEntry("Command"));
}

void MainWindow::conditionRadioButtonToggled(bool checked) {
	if (!checked)
		return;

	bool radioButtonChecked = gui.radioButtonCPU->isChecked();
	if (gui.frameCPU->isVisible() != radioButtonChecked)
		gui.frameCPU->setVisible(radioButtonChecked);
	radioButtonChecked = gui.radioButtonMemory->isChecked();
	if (gui.frameMemory->isVisible() != radioButtonChecked)
		gui.frameMemory->setVisible(radioButtonChecked);
	radioButtonChecked = gui.radioButtonNetwork->isChecked();
	if (gui.frameNetwork->isVisible() != radioButtonChecked) {
		gui.frameNetwork->setVisible(radioButtonChecked);

		if (radioButtonChecked) {
			// Populating the combobox
			gui.comboBoxNetworkInterface->clear();
			QList<QNetworkInterface> interfaces = QNetworkInterface::allInterfaces();
			int i;
			for (i = 0; i < interfaces.size(); i++) {
				const QNetworkInterface interface = interfaces[i];
				if (interface.flags() & QNetworkInterface::IsUp)
					gui.comboBoxNetworkInterface->addItem(interface.name());
			}
			// Loading config for the combobox (just the first time)
			static bool doneOnce = false;
			if (!doneOnce) {
				QString name;
				if (cmdLineArgs->isSet("interface"))
					name = cmdLineArgs->getOption("interface");
				else {
					KConfigGroup groupNetwork = config->group("ConditionNetwork");
#ifdef Q_OS_FREEBSD
					name = groupNetwork.readEntry("Interface", "em0");
#else
					name = groupNetwork.readEntry("Interface", "eth0");
#endif
				}
				i = gui.comboBoxNetworkInterface->findText(name);
				if (i != -1)
					gui.comboBoxNetworkInterface->setCurrentIndex(i);

				doneOnce = true;
			}
		}
	}
	radioButtonChecked = gui.radioButtonDateTime->isChecked();
	if (gui.frameDateTime->isVisible() != radioButtonChecked)
		gui.frameDateTime->setVisible(radioButtonChecked);
	radioButtonChecked = gui.radioButtonProcessDies->isChecked();
	if (gui.frameProcessDies->isVisible() != radioButtonChecked)
		gui.frameProcessDies->setVisible(radioButtonChecked);
}

void MainWindow::actionRadioButtonToggled(bool checked) {
	if (!checked)
		return;

	bool radioButtonChecked = gui.radioButtonShutdown->isChecked()
			|| gui.radioButtonReboot->isChecked();
	if (gui.frameShutdown->isVisible() != radioButtonChecked)
		gui.frameShutdown->setVisible(radioButtonChecked);
	radioButtonChecked = gui.radioButtonSleep->isChecked();
	if (gui.frameSleep->isVisible() != radioButtonChecked)
		gui.frameSleep->setVisible(radioButtonChecked);
	radioButtonChecked = gui.radioButtonExecuteCommand->isChecked();
	if (gui.frameExecuteCommand->isVisible() != radioButtonChecked)
		gui.frameExecuteCommand->setVisible(radioButtonChecked);
	radioButtonChecked = gui.radioButtonPlayAlarm->isChecked();
	if (gui.framePlayAlarm->isVisible() != radioButtonChecked) {
		gui.framePlayAlarm->setVisible(radioButtonChecked);

		static bool doneOnce = false;
		if (!doneOnce && radioButtonChecked) {
			availableSounds = new QStringList(KGlobal::dirs()->findAllResources("sound",
					"Sentinella/*", KStandardDirs::NoDuplicates));
			availableSounds->sort();
			// Loading config for the combobox
			KConfigGroup groupPlayAlarm = config->group("ActionPlayAlarm");
			QString savedFileName = groupPlayAlarm.readEntry("File");
			int savedIndex = -1;
			// Populating the combobox
			for (int i = 0; i < availableSounds->size(); i++) {
				if (savedFileName.compare(availableSounds->at(i)) == 0)
					savedIndex = i;
				QFileInfo fileInfo = QFileInfo(availableSounds->at(i));
				gui.comboBoxPlayAlarm->addItem(fileInfo.baseName());
			}

			if (savedIndex != -1)
				gui.comboBoxPlayAlarm->setCurrentIndex(savedIndex);
			doneOnce = true;
		}
	}
	radioButtonChecked = gui.radioButtonKillProcess->isChecked();
	if (gui.frameKillProcess->isVisible() != radioButtonChecked)
		gui.frameKillProcess->setVisible(radioButtonChecked);
}

void MainWindow::chooseProcess(bool isAction) {
	KDialog* procsDialog = new KDialog(this);
	procsDialog->setCaption(i18nc("@title:window", "Choose a program"));
	procsDialog->setButtons(KDialog::Ok | KDialog::Cancel);

	KSysGuardProcessList* processList = new KSysGuardProcessList(procsDialog);
	processList->setKillButtonVisible(false);
	if (isAction)
		processList->setState(ProcessFilter::OwnProcesses);
	processList->treeView()->setSelectionMode(QAbstractItemView::SingleSelection);
	procsDialog->setMainWidget(processList);

	if (procsDialog->exec() == QDialog::Accepted) {
		QList<KSysGuard::Process*> selectedList = processList->selectedProcesses();
		if (!selectedList.isEmpty()) {
			KSysGuard::Process* selected = selectedList.at(0);
			int index = selected->name.indexOf(' ');
			if (isAction) {
				pidKillProcess = selected->pid;
				gui.labelKillProcess->setText(index == -1 ? selected->name : selected->name.left(
						index));
			} else {
				pidProcessDies = selected->pid;
				gui.labelProcessDies->setText(index == -1 ? selected->name : selected->name.left(
						index));
			}
		}
	}

	delete processList;
	delete procsDialog;
}

void MainWindow::programDiesClicked() {
	chooseProcess(false);
}

void MainWindow::killProcessClicked() {
	chooseProcess(true);
}

void MainWindow::startStopClicked() {
	if (condition == NULL)
		startClicked();
	else
		stopClicked();
}

void MainWindow::startClicked() {
	// UI validation
	if (gui.radioButtonSleep->isChecked()) {
		if (!gui.radioButtonSleepStandBy->isChecked() && !gui.radioButtonSleepSuspend->isChecked()
				&& !gui.radioButtonSleepHibernate->isChecked()) {
			KMessageBox::sorry(this, i18nc(
					"@info The user has to choose between hibernation, standby and Suspend modes",
					"You must select sleep mode."));
			return;
		}
	} else if (gui.radioButtonExecuteCommand->isChecked()) {
		if (gui.lineEditExecuteCommand->text().isEmpty()) {
			KMessageBox::sorry(this, i18n("A command must be specified."));
			return;
		}
	} else if (gui.radioButtonKillProcess->isChecked()) {
		if (pidKillProcess == 0) {
			KMessageBox::sorry(this, i18n("You must select a program to kill."));
			return;
		}
	}
	// TODO Check if the chosen processes to watch for still exists

	if (gui.radioButtonCPU->isChecked()) {
		QTime time = gui.timeEditCPU->time();
		condition = new CPU(time.minute() * 60 + time.second(), 2, gui.spinBoxCPU->value(),
				gui.comboBoxCPU->currentIndex() == 0);
	} else if (gui.radioButtonMemory->isChecked()) {
		QTime time = gui.timeEditMemory->time();
		condition = new Memory(time.minute() * 60 + time.second(), 4, gui.spinBoxMemory->value()
				* 1048576, gui.comboBoxMemory->currentIndex() == 0);
	} else if (gui.radioButtonNetwork->isChecked()) {
		QTime time = gui.timeEditNetwork->time();
		condition = new Network(time.minute() * 60 + time.second(), 2,
				gui.comboBoxNetworkInterface->currentText(), gui.spinBoxNetwork->value() * 1024,
				gui.comboBoxNetworkHigher->currentIndex() == 0,
				gui.comboBoxNetworkDirection->currentIndex() == 0 ? DOWNLOAD : UPLOAD);
	} else if (gui.radioButtonDateTime->isChecked()) {
		int remainingSeconds = QDateTime::currentDateTime().secsTo(
				gui.kdatetimewidgetDateTime->dateTime());
		if (remainingSeconds <= 0) {
			KMessageBox::sorry(this, i18nc("@info",
					"You have selected a date in the past. Please input a future date."));
			return;
		}
		condition = new DateTime(remainingSeconds);
	} else if (gui.radioButtonProcessDies->isChecked()) {
		if (pidProcessDies == 0) {
			KMessageBox::sorry(this, i18nc("@info", "You must select a program to monitor."));
			return;
		}
		condition = new ProcessDies(2, 2, pidProcessDies);
	}

	connect(condition, SIGNAL(finished()), this, SLOT(conditionFinished()));
	condition->start(QThread::HighPriority);
	gui.groupBoxConditions->setEnabled(false);
	gui.groupBoxActions->setEnabled(false);
	gui.pushButtonStartStop->setText(i18nc("@action:button Stop the running task", "Stop"));
	saveConfig();
}

void MainWindow::saveConfig() {
	KConfigGroup group = config->group("Conditions");
	if (gui.radioButtonCPU->isChecked()) {
		group.writeEntry("LastExecuted", 1);
		KConfigGroup groupCPU = config->group("ConditionCPU");
		groupCPU.writeEntry("Higher", gui.comboBoxCPU->currentIndex());
		groupCPU.writeEntry("Percentage", gui.spinBoxCPU->value());
		groupCPU.writeEntry("Time", gui.timeEditCPU->time().toString());
	} else if (gui.radioButtonMemory->isChecked()) {
		group.writeEntry("LastExecuted", 2);
		KConfigGroup groupMemory = config->group("ConditionMemory");
		groupMemory.writeEntry("Higher", gui.comboBoxMemory->currentIndex());
		groupMemory.writeEntry("Amount", gui.spinBoxMemory->value());
		groupMemory.writeEntry("Time", gui.timeEditMemory->time().toString());
	} else if (gui.radioButtonNetwork->isChecked()) {
		group.writeEntry("LastExecuted", 3);
		KConfigGroup groupNetwork = config->group("ConditionNetwork");
		groupNetwork.writeEntry("Direction", gui.comboBoxNetworkDirection->currentIndex());
		groupNetwork.writeEntry("Higher", gui.comboBoxNetworkHigher->currentIndex());
		groupNetwork.writeEntry("Amount", gui.spinBoxNetwork->value());
		groupNetwork.writeEntry("Interface", gui.comboBoxNetworkInterface->currentText());
		groupNetwork.writeEntry("Time", gui.timeEditNetwork->time().toString());
	} else if (gui.radioButtonDateTime->isChecked()) {
		group.writeEntry("LastExecuted", 4);
		KConfigGroup groupDateTime = config->group("ConditionDateTime");
		groupDateTime.writeEntry("DateTime", gui.kdatetimewidgetDateTime->dateTime());
	} else if (gui.radioButtonProcessDies->isChecked())
		group.writeEntry("LastExecuted", 5);

	group = config->group("Actions");
	if (gui.radioButtonShutdown->isChecked()) {
		group.writeEntry("LastExecuted", 1);
		KConfigGroup groupShutdown = config->group("ActionShutdown");
		groupShutdown.writeEntry("Force", gui.checkBoxShutdownForce->isChecked());
	} else if (gui.radioButtonReboot->isChecked()) {
		group.writeEntry("LastExecuted", 2);
		KConfigGroup groupShutdown = config->group("ActionShutdown");
		groupShutdown.writeEntry("Force", gui.checkBoxShutdownForce->isChecked());
	} else if (gui.radioButtonSleep->isChecked()) {
		Solid::PowerManagement::SleepState state;
		group.writeEntry("LastExecuted", 3);
		KConfigGroup groupSleep = config->group("ActionSleep");
		if (gui.radioButtonSleepStandBy->isChecked())
			state = Solid::PowerManagement::StandbyState;
		else if (gui.radioButtonSleepSuspend->isChecked())
			state = Solid::PowerManagement::SuspendState;
		else
			state = Solid::PowerManagement::HibernateState;
		groupSleep.writeEntry("Mode", (int) state);
	} else if (gui.radioButtonExecuteCommand->isChecked()) {
		group.writeEntry("LastExecuted", 4);
		KConfigGroup groupExecuteCommand = config->group("ActionExecuteCommand");
		groupExecuteCommand.writeEntry("Command", gui.lineEditExecuteCommand->text());
	} else if (gui.radioButtonPlayAlarm->isChecked()) {
		group.writeEntry("LastExecuted", 5);
		KConfigGroup groupPlayAlarm = config->group("ActionPlayAlarm");
		groupPlayAlarm.writeEntry("File",
				availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
	} else if (gui.radioButtonKillProcess->isChecked())
		group.writeEntry("LastExecuted", 6);
	config->sync();
}

void MainWindow::stopClicked() {
	disconnect(condition, SIGNAL(finished()), this, SLOT(conditionFinished()));
	condition->aborted = 1;
	condition = NULL; // to avoid this mem leak we can connect the old qthread's finished signal to a method that destroy it
	gui.groupBoxConditions->setEnabled(true);
	gui.groupBoxActions->setEnabled(true);
	gui.pushButtonStartStop->setText(i18nc("@action:button Verb in imperative form", "Start"));
}

void MainWindow::conditionFinished() {
#define PASSIVE_POPUP_TITLE i18nc("@info The name of the app in the title of a passive popup", "Sentinella")
	int error = condition->error;
	delete condition;
	condition = NULL;

	if (error != 0) {
		showError(error);
	} else {
		if (gui.radioButtonShutdown->isChecked()) {
			action = new Shutdown(KWorkSpace::ShutdownTypeHalt,
					gui.checkBoxShutdownForce->isChecked() ? KWorkSpace::ShutdownModeForceNow
							: KWorkSpace::ShutdownModeInteractive);
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Halting the system."), this);
		} else if (gui.radioButtonReboot->isChecked()) {
			action = new Shutdown(KWorkSpace::ShutdownTypeReboot,
					gui.checkBoxShutdownForce->isChecked() ? KWorkSpace::ShutdownModeForceNow
							: KWorkSpace::ShutdownModeInteractive);
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Rebooting the system."), this);
		} else if (gui.radioButtonSleep->isChecked()) {
			Solid::PowerManagement::SleepState state;
			if (gui.radioButtonSleepStandBy->isChecked()) {
				state = Solid::PowerManagement::StandbyState;
				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
						"Condition met. Setting the system's state to standby."), this);
			} else if (gui.radioButtonSleepSuspend->isChecked()) {
				state = Solid::PowerManagement::SuspendState;
				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
						"Condition met. Suspending the system."), this);
			} else {
				state = Solid::PowerManagement::HibernateState;
				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
						"Condition met. Hibernating the system."), this);
			}
			action = new Sleep(state);
		} else if (gui.radioButtonExecuteCommand->isChecked()) {
			action = new ExecuteCommand(gui.lineEditExecuteCommand->text());
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Executing command."), this);
		} else if (gui.radioButtonPlayAlarm->isChecked()) {
			action = new PlayAlarm(availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Throwing alarm."), this);
		} else {
			action = new KillProcess(pidKillProcess);
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Killing the selected program."), this);
		}

		action->execute();

		if (gui.radioButtonPlayAlarm->isChecked()) {
			KMessageBox::information(this, i18n("Close this message to turn off the alarm."));
			((PlayAlarm*) action)->stop();
		}
	}

	gui.groupBoxConditions->setEnabled(true);
	gui.groupBoxActions->setEnabled(true);
	gui.pushButtonStartStop->setText(i18nc("@action:button Verb in imperative form", "Start"));
}

void MainWindow::showError(const int error) {
	switch (error) {
	case ENOSYS:
		KMessageBox::error(this, i18n("Feature not available."));
		break;
	case ENODEV:
		KMessageBox::error(this, i18n("No such device."));
		break;
	case ENOMEM:
		KMessageBox::error(this, i18n("Not enough memory."));
		break;
	case ESRCH:
		KMessageBox::error(this, i18n("No such process."));
		break;
	case EIO:
		KMessageBox::error(this, i18n("I/O error."));
		break;
	default:
		KMessageBox::error(this, i18n("An internal error occurred."));
	}
}

void MainWindow::showAboutApp() {
	KAboutApplicationDialog* aboutAppDialog = new KAboutApplicationDialog(&aboutData, this);
	aboutAppDialog->exec();
	delete aboutAppDialog;
}

void MainWindow::closeEvent(QCloseEvent *event) {
	if (event->spontaneous()) {
		trayIcon.activate();
		event->ignore();
	} else {
		event->accept();
	}
}
